#!/usr/bin/env python

# Terrain + interface
# monkeycow3k

from mediator import Event, mediator
import main
import math
import numpy
import player
import pygame
import pygame.locals
import terrain_colour
import turn

WINSIZE = [640, 480]
MAX_ALTERATION = 100
BRUSH_RADIUS = 40.0
BRUSH_HEIGHT = 12.0

class Terrain(object):    
    def __init__(self, win_size):
        self.win_size = win_size
        self.t = numpy.zeros(self.win_size, dtype='int')
        self.surface = pygame.Surface(self.win_size)
        self.surface.fill(terrain_colour.ColourManager.get_level_colour(0))
        self.dirty_rects = []
        mediator.addListener("tf_start_alter", self.start_listener)
        mediator.addListener("tf_end_alter", self.end_listener)
        self.dir = 0

    def alter(self, pos, dir):
        x,y = pos
        usable_x_min = int(x-BRUSH_RADIUS)
        usable_x_max = int(x+BRUSH_RADIUS)
        usable_y_min = int(y-BRUSH_RADIUS)
        usable_y_max = int(y+BRUSH_RADIUS)
        if(usable_x_min < 0):
            usable_x_min = 0
        if(usable_x_max >= self.win_size[0]):
            usable_x_max = self.win_size[0]-1
        if(usable_y_min < 0):
            usable_y_min = 0
        if(usable_y_max >= self.win_size[1]):
            usable_y_max = self.win_size[1]-1
        brush = pygame.Rect(usable_x_min, usable_y_min, usable_x_max-usable_x_min+1, usable_y_max-usable_y_min+1)
        for x_pos in range(usable_x_min,usable_x_max+1):
            for y_pos in range(usable_y_min,usable_y_max+1):
                dx = x-x_pos
                dy = y-y_pos
                distance = math.sqrt(dx*dx + dy*dy)
                if distance <= BRUSH_RADIUS:
                    adjustment = BRUSH_HEIGHT * (BRUSH_RADIUS - distance) / BRUSH_RADIUS
                    adjustment = math.ceil(adjustment)
                    self.adjust_point(x_pos, y_pos, adjustment, dir)
        self.dirty_rects.append(brush)
        
    def adjust_point(self,x,y,adjustment,dir):
        current_height = self.t[x][y]
        new_height = current_height + adjustment * (1 - 2 * dir)
        if new_height > MAX_ALTERATION:
            new_height = MAX_ALTERATION
        elif new_height < (-1 * MAX_ALTERATION):
            new_height = -1 * MAX_ALTERATION
        self.t[x][y] = new_height

    def _draw(self, rect):
        x1 = rect.x
        x2 = rect.x + rect.w
        y1 = rect.y
        y2 = rect.y + rect.h
        for y in range(y1,y2):
            for x in range(x1,x2):
                self.surface.set_at((x,y), terrain_colour.ColourManager.get_level_colour(self.t[x][y]))
        return
    
    def render(self):
        for rect in self.dirty_rects:
            self._draw(rect)
        self.dirty_rects = []
        return self.surface
    
    def start_listener(self, event):
        mediator.addListener("mouse_pos", self.continue_listener)
        print "terrain.start"
        self.dir = event.button == 3
        
    def end_listener(self, event):
        mediator.removeListener("mouse_pos", self.continue_listener)
        print "terrain.stop"
        
    def continue_listener(self, event):
        print "terrain.alter"
        x,y = event.pos
        mid = float(main.H) / float(main.W) * x
        if ((turn.current_turn.player == player.player0) and (y > mid)) or \
           ((turn.current_turn.player == player.player1) and (y < mid)):
            self.alter(event.pos, self.dir)
            mediator.publishEvent("terrain.alter")

def run_test():
    pygame.init()
    pygame.display.set_caption('Terrain')
    screen = pygame.display.set_mode(WINSIZE)
    clock = pygame.time.Clock()
    
    terrain = Terrain(WINSIZE)
    
    screen.blit(terrain.render(), (0,0))
    pygame.display.update()
    mouse_down = False
    direction = 0
    
    done = False
    while not done:
        clock.tick(15)
        screen.blit(terrain.render(), (0,0))
        pygame.display.update()
        if mouse_down:
            # Affect terrain
            terrain.alter(pygame.mouse.get_pos(), direction)
            screen.blit(terrain.render(), (0,0))
            pygame.display.update()           
        for e in pygame.event.get():
            if e.type == pygame.locals.QUIT or (e.type == pygame.locals.KEYUP and e.key == pygame.locals.K_ESCAPE):
                done = True
                break
            elif e.type == pygame.locals.MOUSEBUTTONDOWN and e.button in [1,3]:
                mouse_down = True
                direction = e.button == 3
            elif e.type == pygame.locals.MOUSEBUTTONUP and e.button in [1,3]:
                mouse_down = False
    return
    
if __name__ == '__main__': run_test()